import React, { Component } from "react";
import { withRouter } from "react-router-dom";
import { func, string } from "prop-types";

import InputComponent from "../common";

class SignIn extends Component {
	static propTypes = {
		token: string,
		auth: func
	};

	componentDidMount() {
		this.loadJS("https://apis.google.com/js/platform.js?onload=init");
	}

	state = {
		singIn: {
			email: {
				value: "",
				config: {
					type: "email",
					name: "email",
					placeholder: "Enter your email"
				},
				valid: false,
				touch: false,
				validation: {
					required: true
				}
			},
			password: {
				value: "",
				config: {
					type: "password",
					name: "password",
					placeholder: "Enter your password"
				},
				valid: false,
				touch: false,
				validation: {
					required: true,
					minLength: 5
				}
			}
		},
		isValid: false,
		token: null
	};

	loadJS = src => {
		const body = window.document.getElementsByTagName("body")[0];
		const ref = body.getElementsByTagName("script")[0];

		const script = window.document.createElement("script");
		script.src = src;
		script.async = true;
		script.defer = true;
		script.onload = () => {
			window.gapi.load("auth2", () => {
				console.log("success");
			});
		};
		ref.parentNode.insertBefore(script, ref);
	};

	submit = e => {
		e.preventDefault();
		const { singIn } = this.state;
		const { auth } = this.props;
		const values = Object.keys(singIn).reduce((prev, elem) => ({ ...prev, [elem]: singIn[elem].value }), {});

		auth(values);
	};

	validator = (value, rules) => {
		let isValid = true;

		if (rules.required) {
			isValid = value.trim() !== "" && isValid;
		}

		if (rules.minLength) {
			isValid = value.length >= rules.minLength && isValid;
		}

		return isValid;
	};

	change = e => {
		const { name, value } = e.target;

		this.setState(prevState => {
			const valid = this.validator(value, prevState.singIn[name].validation);

			const otherValid = Object.keys(prevState.singIn).some(el => !prevState.singIn[el].valid);

			return {
				singIn: {
					...prevState.singIn,
					[name]: {
						...prevState.singIn[name],
						value,
						touch: true,
						valid
					}
				},
				isValid: otherValid && valid
			};
		});
	};

	googleAuth = () => {
		const ga = window.gapi.auth2.init({
			client_id: "665483657556-2qrnlhpqcp3ap5j5kq4u1eas2a0o3h23.apps.googleusercontent.com"
		});

		ga.signIn().then(info => {
			const { Zi } = info;
			console.log("p1", info);
			fetch("http://localhost:5000/auth/google", {
				method: "post",
				headers: {
					"Content-Type": "application/json"
				},
				body: JSON.stringify({ access_token: Zi.access_token })
			})
				.then(res => res.json())
				.then(res => console.log("res", res))
				.catch(err => console.log(err.response));
		});
	};

	render() {
		const { singIn, isValid } = this.state;
		return (
			<form className="auth-box__sign-in-form" onSubmit={this.submit}>
				{Object.keys(singIn).map(el => (
					<InputComponent
						key={el}
						touch={singIn[el].touch}
						valid={!singIn[el].valid}
						config={{ ...singIn[el].config, value: singIn[el].value, onChange: this.change }}
					/>
				))}
				<button type="button" onClick={this.googleAuth}>
					Google link
				</button>
				<a href="https://github.com/login/oauth/authorize?client_id=115f273c8312fc176907">GitHub link</a>
				<button type="submit">Sign in</button>
			</form>
		);
	}
}

export default withRouter(SignIn);
